import 'dart:typed_data';

typedef NUIAssetChannel = Future<ByteData> Function(String key);

class NUISystem {
  static Map<String, NUIAssetChannel> _assetChannels = Map();

  static void setAssetChannel(String key, NUIAssetChannel channel){
    _assetChannels[key] = channel;
  }

  static List<NUIAssetChannel> get assetChannels{
    return _assetChannels.values;
  }

  static Future<ByteData> getAsset(String key) async{
    ByteData backupAsset;
    if(_assetChannels != null && _assetChannels.isNotEmpty){
      for(var channel in _assetChannels.values){
        final ass = await channel(key);
        if(ass != null){
          backupAsset = ass;
          break;
        }
      }
    }
    return backupAsset;
  }

  static void removeAssetChannel(String key){
    _assetChannels.remove(key);
  }

  static void clearAssetChannel(){
    _assetChannels.clear();
  }

}