import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';

class NUIDraggableFloatingButtonBean{
  NUIDraggableFloatingButtonBean({@required this.margin, this.initialAlignment : Alignment.bottomRight, this.screenHeight, this.screenWidth, this.top, this.left, this.bottom, this.right});
  final double screenHeight;
  final double screenWidth;
  final Alignment initialAlignment;

  double width;
  double height;

  final EdgeInsets margin;
  double left;
  double right;
  double bottom;
  double top;

  double targetTop;
  double targetLeft;

  void setSize(double width, double height){
    this.width = width;
    this.height = height;
  }

  void initPosition(){
    bottom = null;
    right = null;
    _setToBottomMax();
    _setToRight();

    left = targetLeft;
    top = targetTop;
  }

  void _setToLeft(){
    targetLeft = margin.left;
  }

  void _setToRight(){
    targetLeft = screenWidth - width - margin.right;
  }

  void _setToTopMax(){
    targetTop = margin.top;
  }

  void _setToBottomMax(){
    targetTop = screenHeight - height - margin.bottom;
  }

  void _repositionForY(double y){
    if(y > screenHeight - height - margin.bottom){
      _setToBottomMax();
    }
    else if(y < margin.top){
      _setToTopMax();
    }
    else{
      targetTop = y;
    }
  }

  double getAnimatedLeft(double percent){
    final diff = targetLeft - left;
    final ratio = diff * percent;
    return targetLeft - (diff - ratio);
  }

  double getAnimatedTop(double percent){
    final diff = targetTop - top;
    final ratio = diff * percent;
    return targetTop - (diff - ratio);
  }

  void completeAnimate(){
    top = targetTop;
    left = targetLeft;
  }

  void updatePosition(DraggableDetails info, {bool complete: false}){
    final x = info.offset.dx;
    final y = info.offset.dy;

    final towardsLeft = (x + (width/2)) < screenWidth/2;

    top = y;
    left = x;

    right = null;
    bottom = null;
    _repositionForY(y);

    if(towardsLeft){
      _setToLeft();
    }
    else{
      _setToRight();
    }

    if(complete == true){
      this.completeAnimate();
    }
  }
}

class NUIDraggableFloatingButton extends StatefulWidget {
  NUIDraggableFloatingButton({@required this.child, @required this.width, @required this.height, this.initialAlignment: Alignment.bottomRight, this.feedback, this.childOnDrag, this.margin, this.enabled: true, Key key}) : super(key: key);
  final bool enabled;
  final EdgeInsets margin;
  final Widget child;
  final Widget feedback;
  final Widget childOnDrag;
  final Alignment initialAlignment;
  final double width;
  final double height;

  @override
  _NUIDraggableFloatingButtonState createState() => _NUIDraggableFloatingButtonState();
}

class _NUIDraggableFloatingButtonState extends State<NUIDraggableFloatingButton> with TickerProviderStateMixin {
  AnimationController chatAnimationController;
  bool animatingChat = false;
  Animation<double> animation;
  NUIDraggableFloatingButtonBean bean;

  @override
  void initState(){
    super.initState();
    _initAnimation();
  }

  void _initChatButtonLocation(BuildContext context){
    if(bean != null) return;
    bean = NUIDraggableFloatingButtonBean(
        screenWidth: screenWidth(context),
        screenHeight: screenHeight(context),
        initialAlignment: widget.initialAlignment,
        margin: widget.margin ?? EdgeInsets.all(0)
    );
    bean.setSize(widget.width, widget.height);
    bean.initPosition();
  }

  void _initAnimation(){
    chatAnimationController = new AnimationController(duration: Duration(milliseconds: 500), vsync: this)
      ..addStatusListener((status) {
        if (status == AnimationStatus.dismissed) {
          animatingChat = false;
          bean.completeAnimate();
          chatAnimationController.dispose();
          setState(() {

          });
        }
      });
  }

  void _animateChat(){
    _initAnimation();
    animation = Tween(begin: 0.0, end: 1.0).animate(CurvedAnimation(parent: chatAnimationController, curve: Curves.bounceOut));
    chatAnimationController.forward();
    animatingChat = true;
  }

  @override
  Widget build(BuildContext context) {
    _initChatButtonLocation(context);
    return AnimatedBuilder(
        animation: chatAnimationController,
        builder: (data, context) {
          return Builder(
              builder: (context) {
                return Positioned(
                  right: bean.right,
                  bottom: bean.bottom,
                  top: bean.getAnimatedTop(animation?.value ?? 1),
                  left: bean.getAnimatedLeft(animation?.value ?? 1),
                  child: Draggable<NUIDraggableFloatingButtonBean>(
                    data: bean,
                    child: widget.child,
                    childWhenDragging: widget.childOnDrag ?? SizedBox.shrink(),
                    feedback: widget.feedback ?? widget.child,
                    onDragEnd: (info) {
                      bean.updatePosition(info);
                      _animateChat();
                      setState(() {

                      });
                    },
                  ),
                );
              }
          );
        }
    );
  }
}
